{*****************************************}
{    Visualiseur de Clipboard             }
{    Ralisation : Pascal Peyremorte      }
{    Programme test pour essais...        }
{                                         }
{    p.peyremorte@free.fr                 }
{                                         }
{    Distribution et utilisation LIBRE    }
{*****************************************}

{ 10/06/99  V1.00    Premire construction
  13/03/00  V1.10    Traitement de la taille des donnes}

unit U_ClpBrd;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls;

type
  TEcrPrin = class(TForm)
    Liste: TListBox;
    PanelInfo: TPanel;
    Detail: TListBox;
    Memo1: TMemo;
    Panel1: TPanel;
    procedure ListeClick(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private { Dclarations prives }
    NextViewer:THandle;
    procedure WMChangeCBChain(Var Msg:TMessage); Message WM_CHANGECBCHAIN;
    procedure WMDrawClipBoard(Var Msg:TMessage); Message WM_DRAWCLIPBOARD;
  public  { Dclarations publiques }
  end;

var
  EcrPrin: TEcrPrin;

implementation

{$R *.DFM}

//------------------------------------------
// Creation / destruction / dimentions
//------------------------------------------

procedure TEcrPrin.FormCreate(Sender: TObject);
begin
  NextViewer:=SetClipboardViewer(Handle);
end;

procedure TEcrPrin.FormDestroy(Sender: TObject);
begin
  ChangeClipboardChain(Handle, NextViewer);
end;

procedure TEcrPrin.FormResize(Sender: TObject);
begin
  Detail.Height:=ClientHeight-Liste.Height-4;
  Liste.Width:=ClientWidth-PanelInfo.width;
  ListeClick(Nil);
end;


//------------------------------------------
// Messages spcifiques clipboard
//------------------------------------------

procedure TEcrPrin.WMChangeCBChain(Var Msg:TMessage);
Begin
  If Msg.WParam=NextViewer
    then NextViewer:=Msg.lParam
    else SendMessage(NextViewer, Msg.Msg, Msg.wparam, Msg.lParam);
  Msg.Result:=0;
End;

procedure TEcrPrin.WMDrawClipBoard(Var Msg:TMessage);
Var I,L : Integer;
    Buf : String;
Begin
  If NextViewer<>0 then
    SendMessage(NextViewer, Msg.Msg, Msg.wparam, Msg.lParam);
  Msg.Result:=0;
  If OpenClipboard(Handle) then
  Try
    Liste.Clear;
    Detail.Clear;

    I:=EnumClipboardFormats(0);
    While I<>0 do
    Begin
      SetLength(Buf,256);
      L:=GetClipboardFormatName(I, PChar(Buf), 255);
      SetLength(Buf,L);
      If L=0 then
        Case I of
            1: Buf:='CF_TEXT';
            2: Buf:='CF_BITMAP';
            3: Buf:='CF_METAFILEPICT';
            4: Buf:='CF_SYLK';
            5: Buf:='CF_DIF';
            6: Buf:='CF_TIFF';
            7: Buf:='CF_OEMTEXT';
            8: Buf:='CF_DIB';
            9: Buf:='CF_PALETTE';
           10: Buf:='CF_PENDATA';
           11: Buf:='CF_RIFF';
           12: Buf:='CF_WAVE';
           13: Buf:='CF_UNICODETEXT';
           14: Buf:='CF_ENHMETAFILE';
           15: Buf:='CF_HDROP';
           16: Buf:='CF_LOCALE';
           17: Buf:='CF_MAX';
          128: Buf:='CF_OWNERDISPLAY';
          129: Buf:='CF_DSPTEXT';
          130: Buf:='CF_DSPBITMAP';
          131: Buf:='CF_DSPMETAFILEPICT';
          142: Buf:='CF_DSPENHMETAFILE';

          { "Private" formats don't get GlobalFree()'d }
          512: Buf:='CF_PRIVATEFIRST';
          513..
          766: Buf:='CF_Private unknown';
          767: Buf:='CF_PRIVATELAST';

          { "GDIOBJ" formats do get DeleteObject()'d }
          768: Buf:='CF_GDIOBJFIRST';
          769..
         1022: Buf:='CF_GDI unknown';
         1023: Buf:='CF_GDIOBJLAST';
         else
               Buf:='Unknown format';
        End;
      Buf:=Buf+' ('+IntToStr(GlobalSize(GetClipboardData(I)))+')';
      Liste.Items.AddObject(IntToStr(I)+' : '+Buf, Pointer(I));

      I:=EnumClipboardFormats(I);
    End;
  Finally
    CloseClipboard;
  End
  else
    ShowMessage('Clipboard innaccessible');
End;

procedure TEcrPrin.ListeClick(Sender: TObject);
Var Fmt  : Integer; //Format de la donne demande
    H    : THandle; //Handle des donnes clipboard
    P    : PChar;   //pointeur dans donnes clipboard
    I, J : Integer;
    D,S  : String;  //Dump et String quivalente

    HandlePS : THandle;  //Handle de la structure
begin
  If Liste.ItemIndex<0 then Exit;
  Detail.Items.clear;

  Fmt:=Integer(Liste.Items.Objects[Liste.ItemIndex]);

  If Fmt=CF_OWNERDISPLAY then
  Begin
    HandlePS := GlobalAlloc(GMEM_DDESHARE, sizeof(TPAINTSTRUCT));
    With PPaintStruct(GlobalLock(HandlePS))^ do
    Begin
      hdc:=GetDC(PanelInfo.Handle);
      fErase:=True;
      rcPaint:=Rect(0,0,PanelInfo.Width,PanelInfo.Height);
    End;
    GlobalUnlock(HandlePS);
    SendMessage(GetClipboardOwner, WM_PAINTCLIPBOARD,
                PanelInfo.Handle, HandlePS);
    GlobalFree(HandlePS);
  End;

  If OpenClipboard(Handle) then
  Try
    H:=GetClipboardData(Fmt);
    If H>0 then
    Begin
      P:=GlobalLock(H);
      If P<>Nil then
      Begin
        I:=GlobalSize(H);
        While I>0 do
        Begin
          D:=''; S:='';
          J:=16;
          While (J>0) and (I>0) do
          Begin
            D:=D+IntToHex(Byte(P^),2)+' ';
            If P^>#$20
              then S:=S+P^
              else S:=S+'.';
            Inc(P);
            Dec(I);
            Dec(J);
          End;
          Detail.Items.Add(D+S);
        End
      end
      else
        Detail.Items.Add('Ptr = Nil...');
      GlobalUnlock(H);
    End;
  Finally
    CloseClipboard;
  End;
end;

end.
